/// <reference types="node" />
import * as http from 'http';
import * as WebSocket from 'ws';
import { ConnectionContext } from './NodeConstants';
import { MidleWare } from './util/MidlewareManager';
import { ActionRequest } from './Constants';
export declare type ActionCallback = (data: any, ws: WebSocket, accept: (data?: any) => void, reject: (cause: any, message?: string) => void) => void;
/**
 * Parametros disponíveis no Midleware de Actions
 */
export declare type ActionMidleWareContext = {
    /**
     * A conexão com o Client
     */
    ws: WebSocket;
    /**
     * Dados da requisição
     */
    request: http.IncomingMessage;
    /**
    * Identificador da requisição, permite devolver uma resposta para essa solicitação
    */
    id: string;
    /**
     * Nome da ação invocada
     */
    action: string;
    /**
     * Dados para execução da ação
     */
    data: any;
};
/**
 * Tipagem dos Midlewares de actions
 */
export declare type ActionMidleWare = MidleWare<ActionMidleWareContext>;
/**
 * Actions RPC, permite registrar métodos que podem ser invocados pelo Cliente
 */
declare class Actions {
    private actions;
    private middlewares;
    /**
     * Registra uma nova ação que pode ser executada pelo usuário
     *
     * @param name
     * @param callback
     */
    register(name: string, callback: ActionCallback, middleware?: ActionMidleWare | Array<ActionMidleWare>): void;
    /**
     * Executa uma ação
     *
     * @param actionName
     * @param data
     * @param ws
     */
    exec(context: ConnectionContext, request: ActionRequest): Promise<any>;
    /**
     * Permite adicionar Midlewares na execução das actions
     *
     * @param middlewares Um ou mais midlewares
     * @param action A action é opcional. O Wildcard "*" é usado quando se deseja adicionar um midleware para todas as actions (basta não informar a action)
     */
    use(middlewares: ActionMidleWare | Array<ActionMidleWare>, action?: string): void;
    /**
     * Permite remover um midleware adicionado previamente
     *
     * @param middleware
     * @param action
     */
    removeMidleware(middleware: ActionMidleWare, action?: string): void;
}
declare const _default: Actions;
export default _default;
