"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var MidlewareManager_1 = __importDefault(require("./util/MidlewareManager"));
/**
 * Actions RPC, permite registrar métodos que podem ser invocados pelo Cliente
 */
var Actions = /** @class */ (function () {
    function Actions() {
        this.actions = {};
        this.middlewares = new MidlewareManager_1.default();
    }
    /**
     * Registra uma nova ação que pode ser executada pelo usuário
     *
     * @param name
     * @param callback
     */
    Actions.prototype.register = function (name, callback, middleware) {
        this.actions[name] = callback;
        if (middleware) {
            this.use(middleware, name);
        }
    };
    /**
     * Executa uma ação
     *
     * @param actionName
     * @param data
     * @param ws
     */
    Actions.prototype.exec = function (context, request) {
        var _this = this;
        var actionName = request.action;
        return new Promise(function (accept, reject) {
            if (!_this.actions.hasOwnProperty(actionName)) {
                return reject('Operação não cadastrada');
            }
            var midlContext = {
                ws: context.ws,
                request: context.request,
                id: request.id,
                data: request.data,
                action: request.action
            };
            //  Executa os Midlewares da action informada
            return _this.middlewares.exec(actionName, midlContext)
                .then(function () {
                // Executa a action
                _this.actions[actionName](midlContext.data, midlContext.ws, accept, function (cause, message) {
                    console.error(actionName, cause, message || '');
                    reject(message || cause);
                });
            });
        });
    };
    /**
     * Permite adicionar Midlewares na execução das actions
     *
     * @param middlewares Um ou mais midlewares
     * @param action A action é opcional. O Wildcard "*" é usado quando se deseja adicionar um midleware para todas as actions (basta não informar a action)
     */
    Actions.prototype.use = function (middlewares, action) {
        this.middlewares.add(middlewares, action);
    };
    /**
     * Permite remover um midleware adicionado previamente
     *
     * @param middleware
     * @param action
     */
    Actions.prototype.removeMidleware = function (middleware, action) {
        this.middlewares.remove(middleware, action);
    };
    return Actions;
}());
exports.default = new Actions;
//# sourceMappingURL=Actions.js.map