import ClientStorage from "./storage/ClientStorage";
/**
 * Classe de acesso aos dados.
 *
 * O resultado de todas as mensagens de tópicos são persistidos OFFLINE, com isso, evita tráfego desnecessário.
 *
 * Implementa um mecanismo de sincronização com o servidor, informando ao mesmo sobre a ultima versão recebida de cada tópico
 */
export default class Client {
    private LOGGER;
    /**
     * Identificador das requisições feitas ao servidor
     */
    static REQUEST_SEQUENCE: {
        id: number;
    };
    private ws;
    private reconnect;
    /**
     * Deixa salvo as inscrições em tópicos, em problema de conexão, solicita novamente a subscriçao
     */
    private subscriptions;
    /**
     * Permite receber resposta para uma execução
     */
    private promises;
    private timeouts;
    private host;
    private storage;
    constructor(host: string, storage: ClientStorage);
    /**
     * Quando invocado, não será mais criado novas conexões
     */
    close(): void;
    /**
     * Inicializa a conexão do DataLayer com o Backend
     */
    connect(callback?: (err?: any) => void): void;
    /**
     * Aguarda uma mensagem específica do servidor (exemplo. wsLoginSucces)
     *
     * @param topic
     */
    subscribe(topic: string, callback: (data: any) => void): () => void;
    /**
     * Permite executar alguma ação no server (Ex.: do('login', nomeDeUsuario))
     *
     * @param action
     */
    exec(action: string, data: any): Promise<any>;
    /**
     * Informa ao servidor quais os tópicos que essa conexão está ouvindo
     * Evita que o servidor envie todas as mensagens para todos
     */
    private syncTopics;
    /**
     * Informa ao servidor quais os tópicos que essa conexão está ouvindo
     * Evita que o servidor envie todas as mensagens para todos
     */
    private syncTopic;
    /**
     * Timeouts controlados
     *
     * @param handler
     * @param timeout
     */
    setTimeout(handler: (...args: any[]) => void, timeout: number): number;
    isClosed(): boolean;
}
