import { AnyObject } from './Constants';
import Repository from './repository/Repository';
/**
 * Configurações da consulta usada por um publicador
 */
export interface QueryConfig {
    /**
    * Banco de dados que possui a informação
    */
    repository: Repository<any>;
    /**
     * Os parametros de consulta no store
     *
     * Quando o tópico for por id, pode se usar o formato query:{ nomeCampo : '$id' }. O $id será substituido pelo valor informado na publicação
     */
    params?: AnyObject;
    /**
     * Permite adicionar configurações adicionais que serão avalidadas pelo storage
     */
    options?: AnyObject;
    /**
     * Informa que dever retornar apenas um resultado (findOne)
     */
    singleResult?: boolean;
    /**
     * Após a recuperação, permite adicionar um filtro mais sofisticado
     *
     * Não se aplica quando singleResult=true
     */
    filter?: (row: AnyObject, index: number, rows: Array<AnyObject>, prevResults: Array<any>) => boolean;
    /**
     * Após filtrar os valores, permite visitar cada um dos itens
     *
     * Não se aplica quando singleResult=true
     */
    forEach?: (row: AnyObject, index: number, rows: Array<AnyObject>, prevResults: Array<any>) => void;
    /**
     * Permite mapear o resultado para outra estrutura de dados
     */
    map?: (row: AnyObject, index: number, rows: Array<AnyObject>, prevResults: Array<any>) => any;
    /**
     * Permite extrair o valor de saída, a partir dos resultado atual e anteriores
     */
    extract?: (rows: Array<AnyObject>, prevResults: Array<any>) => AnyObject;
}
/**
 * Configurações de um publisher
 */
export interface Config {
    /**
     * Nome do tópico
     */
    topic: string;
    /**
    * Informa que este tópico é por ID, tópico de detalhamento de um item
    */
    idRequired?: boolean;
    /**
     * A configuração da Query deste publicador.
     *
     * Permite aninhar diversas queries
     */
    query: QueryConfig | Array<QueryConfig>;
    /**
     * Invocado após o envio do tópico
     */
    then?: (lastResult: AnyObject) => void;
}
/**
 * Permite criação e invocação de publicadores
 *
 * @TODO: Permitir adicionar MidleWares nos publicadores
 */
declare class Publishers {
    private publishers;
    private publishersByID;
    /**
     * Faz a publicação em um tópico
     *
     * @param topic
     * @param id
     */
    publish(topic: string, id?: string | number): void;
    /**
     * Registra uma nova ação que pode ser executada pelo usuário
     *
     * @param name
     * @param callback
     */
    create(config: Config): void;
    /**
    * Executa uma query
    *
    * @param id
    * @param prevResults
    * @param config
    */
    private exec;
}
declare const _default: Publishers;
export default _default;
