"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var Topic_1 = __importDefault(require("./Topic"));
/**
 * Limita a execução dos Publisher a no máximo 1 a cada <EXEC_LIMIT_TIMEOUT> ms
 */
var EXEC_LIMIT_TIMEOUT = 100;
/**
 * Permite criação e invocação de publicadores
 *
 * @TODO: Permitir adicionar MidleWares nos publicadores
 */
var Publishers = /** @class */ (function () {
    function Publishers() {
        this.publishers = {};
        // No momento da execução, cria um trotle
        this.publishersByID = {};
    }
    /**
     * Faz a publicação em um tópico
     *
     * @param topic
     * @param id
     */
    Publishers.prototype.publish = function (topic, id) {
        // Tópico não existe, verifica se o nome do topico é valido
        if (!this.publishers.hasOwnProperty(topic)) {
            return;
        }
        this.publishers[topic](id);
    };
    /**
     * Registra uma nova ação que pode ser executada pelo usuário
     *
     * @param name
     * @param callback
     */
    Publishers.prototype.create = function (config) {
        var _this = this;
        var topic = config.topic;
        var byId = config.idRequired;
        var queries;
        if (Array.isArray(config.query)) {
            queries = config.query;
        }
        else {
            queries = [config.query];
        }
        var then = config.then;
        // Registra o nome do tópico, sem isso, não existe instanciação automática
        Topic_1.default.REGISTERED_NAMES.push(topic);
        /**
         * Funcao de publicacao
         *
         * @param id
         */
        var publisher = function (id) {
            var topicName = topic;
            if (byId) {
                if (id === undefined) {
                    console.error(topic + ": ID is required");
                    return;
                }
                topicName = topicName + "#" + id;
            }
            // Na primeira execução do Publisher, cria a função de execução real, limitando as chamadas
            if (!_this.publishersByID[topicName]) {
                // Debounced 
                _this.publishersByID[topicName] = (function () {
                    var timeout;
                    return function () {
                        if (timeout) {
                            // Se for invocado nova chamada para esse cara, cancela o agendamento anterior
                            clearTimeout(timeout);
                        }
                        timeout = setTimeout(function () {
                            var index = 0;
                            var prevResults = [];
                            var next = function () {
                                if (index > queries.length - 1) {
                                    var lastResult = prevResults.pop();
                                    // Sempre entrega algum resultado
                                    if (lastResult === undefined || lastResult === null) {
                                        if (queries[queries.length - 1].singleResult) {
                                            lastResult = null;
                                        }
                                        else {
                                            lastResult = [];
                                        }
                                    }
                                    // Se houver registros, faz envio do tópico
                                    Topic_1.default.send(topicName, lastResult);
                                    if (then) {
                                        then(lastResult);
                                    }
                                }
                                else {
                                    // Executa proxima query
                                    _this.exec(id, prevResults, queries[index++])
                                        .then(function (result) {
                                        prevResults.push(result);
                                    })
                                        .then(next)
                                        .catch(function (err) {
                                        console.error("" + topicName, err);
                                    });
                                }
                            };
                            // Inicio da execução
                            next();
                        }, EXEC_LIMIT_TIMEOUT);
                    };
                })();
            }
            // Solicita execução do throtle
            _this.publishersByID[topicName]();
        };
        this.publishers[topic] = publisher;
    };
    /**
    * Executa uma query
    *
    * @param id
    * @param prevResults
    * @param config
    */
    Publishers.prototype.exec = function (id, prevResults, config) {
        var repository = config.repository;
        var params = __assign({}, (config.params || {}));
        var options = __assign({}, (config.options || {}));
        var filter = config.filter;
        var forEach = config.forEach;
        var map = config.map;
        var extract = config.extract;
        return new Promise(function (accept, reject) {
            changeQueryIdValue(params, id);
            if (config.singleResult) {
                repository.findOne(params, options)
                    .then(function (doc) {
                    if (map && doc !== undefined) {
                        doc = map(doc, 0, [doc], prevResults);
                    }
                    if (extract && doc !== undefined) {
                        doc = extract([doc], prevResults);
                    }
                    accept(doc !== undefined ? [doc] : []);
                })
                    .catch(reject);
            }
            else {
                repository.find(params, options)
                    .then(function (docs) {
                    if (docs) {
                        if (filter) {
                            docs = docs.filter(function (value, index, array) {
                                return filter(value, index, array, prevResults);
                            });
                        }
                        if (forEach) {
                            docs.forEach(function (value, index, array) {
                                forEach(value, index, array, prevResults);
                            });
                        }
                        if (map) {
                            docs = docs.map(function (value, index, array) {
                                return map(value, index, array, prevResults);
                            });
                        }
                        if (extract) {
                            docs = [extract(docs, prevResults)];
                        }
                    }
                    accept(docs || []);
                })
                    .catch(reject);
            }
        });
    };
    return Publishers;
}());
/**
 * Substitui em tempo de execução o $id informado
 *
 * @param query
 * @param id
 */
var changeQueryIdValue = function (query, id) {
    if (Array.isArray(query)) {
        query.forEach(function (item) { return changeQueryIdValue(item, id); });
    }
    else if (typeof query === 'object') {
        for (var a in query) {
            if (!query.hasOwnProperty(a)) {
                continue;
            }
            if (query[a] === '$id') {
                query[a] = id;
            }
            else if (Array.isArray(query[a])) {
                changeQueryIdValue(query[a], id);
            }
            else if (typeof query[a] === 'object') {
                changeQueryIdValue(query[a], id);
            }
        }
    }
};
exports.default = new Publishers;
//# sourceMappingURL=Publishers.js.map