/// <reference types="node" />
import * as http from 'http';
import * as WebSocket from 'ws';
import { MidleWare } from './util/MidlewareManager';
import './actions/syncTopicAction';
import './actions/syncTopicsAction';
/**
 * Contexto disponíveis no Midleware do server
 */
export declare type ServerMidleWareContext = {
    /**
     * A conexão com o Client
     */
    ws: WebSocket;
    /**
     * Dados da requisição
     */
    request: http.IncomingMessage;
    /**
     * Evento que originou a execução desse midleware
     */
    event: 'message' | 'close' | 'connection' | 'upgrade';
};
/**
 * Contexto disponíveis no Midleware de Mensagens
 */
export declare type ServerMidleWareMessageContext = ServerMidleWareContext & {
    /**
     * Dados da meensagem.
     */
    data: WebSocket.Data;
};
/**
 * Contexto disponíveis no Midleware de fechamento de conexões
 */
export declare type ServerMidleWareCloseContext = ServerMidleWareContext & {
    /**
      * Código da desconexão.
      */
    code: number;
    /**
     * Razão da desconexão.
     */
    reason: string;
};
/**
 * Os eventos possíveis de uso de Midlewares no server
 */
export declare type ServerMidleWareEvent = 'message' | 'close' | 'connection' | 'upgrade' | '*';
/**
 * Tipagem dos Midlewares de actions
 */
export declare type ServerMidleWare = MidleWare<ServerMidleWareContext | ServerMidleWareMessageContext | ServerMidleWareCloseContext>;
/**
 * Servidor do Websocket-pubsub, faz gerenciamento de conexões e todo o tratamento necessário para o funcionamento da biblioteca
 */
export default class Server {
    server: http.Server;
    webSocketServer: WebSocket.Server;
    private middlewares;
    constructor(server: http.Server, options?: WebSocket.ServerOptions, callback?: () => void);
    /**
     * Finaliza o webSocketServer e o http.Server usados
     *
     * @param cb
     */
    close(cb?: (err?: Error) => void): void;
    /**
     * Permite adicionar um MidleWare na conexão.
     *
     * Util para aplicar autenticação e outros controles
     *
     * @param middleware
     */
    use(middleware: ServerMidleWare | Array<ServerMidleWare>, event?: ServerMidleWareEvent): void;
    /**
     * Permite remover um Midleware adicionado anteriormente
     *
     * @param event
     * @param middleware
     */
    removeMidleware(middleware: ServerMidleWare, event?: ServerMidleWareEvent): void;
    private onConnection;
    private createHandleOnUpgrade;
    private createHandleOnClose;
    private createHandleOnMessage;
    /**
     * Faz a execução dos Midelewares e Tratamento de um action
     */
    private execAction;
}
