import ClientStorage from "./ClientStorage";
/**
 * Implementação abstrata com cache em memória para agilizar o acesso aos dados do storage.
 *
 * Os dados ficam em cache por 10 segundos apenas
 */
export default abstract class ClientStorageAbstract implements ClientStorage {
    protected namespace: string;
    /**
    * Cache dos dados, serializado para garantir a imutalidade
    */
    private cache;
    private promises;
    private timeouts;
    constructor(namespace: string);
    abstract keys(): Promise<Array<string>>;
    protected abstract getItem(namespaceKey: string): Promise<string | null>;
    protected abstract setItem(namespaceKey: string, data: string): Promise<void>;
    protected abstract removeItem(namespaceKey: string): Promise<void>;
    /**
     * Atualiza o prazo de vida de um item do cache
     *
     * @param key
     */
    touch(key: string): void;
    get<T>(key: string): Promise<T>;
    set(key: string, data: any): Promise<void>;
    toNamespaceKey(key: string): string;
    remove(key: string): Promise<void>;
}
