"use strict";
// IGNORA
//  34 = '"'
//  92 = '\'
// 124 = '|'
Object.defineProperty(exports, "__esModule", { value: true });
var PRIMITIVES = {
    'true': true,
    'false': false,
    'null': null
};
/**
 * As chaves já conhecidas, evita trafego desnecessário
 */
var DEFAULT_KEYS = [
    ' ', '{', '}', '|', ':',
    '@',
    'a', 'm', 'd', 'r',
    '0', '1', '2', '3', '4', '5', '6', '7', '8', '9',
    0, 1, 2, 3, 4, 5, 6, 7, 8, 9,
    '$id', '_id', 'id', 'key', '_key',
    true, false, null,
    'true', 'false', 'null'
];
var SPACE = fromIndex(DEFAULT_KEYS.indexOf(' '));
var OBJECT_OPEN = fromIndex(DEFAULT_KEYS.indexOf('{'));
var OBJECT_CLOSE = fromIndex(DEFAULT_KEYS.indexOf('}'));
/**
 * Obtém o caractere que representa o índice informado
 *
 * @param index
 */
function fromIndex(index) {
    if (index >= 124) {
        // Ignora caracteres "|" e "\"
        index += 2;
    }
    else if (index >= 92) {
        // Ignora caractere "\"
        index++;
    }
    return String.fromCharCode(index + 35);
}
/**
 * Obtém o índice representado pleo caractere
 *
 * @param str
 */
function toIndex(str) {
    var charCode = str.charCodeAt(0);
    if (charCode > 124) {
        // Restaura caracteres "|" e "\"
        charCode -= 2;
    }
    else if (charCode > 92) {
        // Restaura caractere "\"
        charCode--;
    }
    return charCode - 35;
}
/**
 * Compactação do objeto para transporte
 *
 * O objeto precisa ter sido tratado com o DTO::flatten
 *
 * @param flatObj
 */
function compress(flatObj) {
    var keys = DEFAULT_KEYS.slice(0);
    // Quando o tipo do dado é dúbil (ex 1.21 pode ser "1.21" ou um float, ou true, pode ser "true" ou o booleano)
    // O índice é salvo
    var markedAsString = [];
    var output = '';
    /**
     * Marca o valor como string, para evitar tratamento de primitivos no decompress
     *
     * @param {*} value
     */
    function markAsString(value) {
        if (typeof value === 'string') {
            if (value.match(/^[-+]?\d+$/)) {
                markedAsString.push(fromIndex(output.length));
            }
            else if (value.match(/^[-+]?[\d.,]+$/)) {
                markedAsString.push(fromIndex(output.length));
            }
            else if (['true', 'false', 'null'].indexOf(value) >= 0) {
                markedAsString.push(fromIndex(output.length));
            }
        }
    }
    function parse(obj) {
        var type = typeof obj;
        if (type === 'string') {
            var words_1 = obj.split(' ');
            words_1.forEach(function (str, index) {
                if (str !== '') {
                    var keyIdx = keys.indexOf(str);
                    if (keyIdx < 0) {
                        keyIdx = keys.push(str) - 1;
                    }
                    output += fromIndex(keyIdx);
                    markAsString(str);
                }
                if (index < words_1.length - 1) {
                    output += SPACE;
                }
            });
        }
        else if (type === 'number' || type === 'boolean' || type === 'symbol' || obj === null) {
            var keyIdx = keys.indexOf(obj);
            if (keyIdx < 0) {
                keyIdx = keys.push(obj) - 1;
            }
            output += fromIndex(keyIdx);
        }
        else if (Array.isArray(obj)) {
            throw new Error('Array não é permitido. Objeto deve ser tratado com o DTO.flatten() antes de acionar esse método.');
        }
        else if (typeof obj === 'object') {
            output += OBJECT_OPEN;
            for (var key in obj) {
                if (!obj.hasOwnProperty(key)) {
                    continue;
                }
                var keyIdx = keys.indexOf(key);
                if (keyIdx < 0) {
                    keyIdx = keys.push(key) - 1;
                }
                output += fromIndex(keyIdx);
                parse(obj[key]);
            }
            output += OBJECT_CLOSE;
        }
        // else {
        //    let valIdx = keys.indexOf(obj);
        //    if (valIdx < 0) {
        //       valIdx = keys.push(obj) - 1;
        //    }
        //    output += fromIndex(valIdx);
        //    markAsString(obj);
        // }
    }
    parse(flatObj);
    // Remove os caracteres conhecidos
    keys.splice(0, DEFAULT_KEYS.length);
    var header = '';
    if (markedAsString.length > 0) {
        markedAsString.forEach(function (idx) {
            header += ("" + idx).replace(/[|]/g, '\\|') + '|';
        });
        header += '|';
    }
    else {
        header += '||';
    }
    if (keys.length > 0) {
        keys.forEach(function (key) {
            header += ("" + key).replace(/[|]/g, '\\|') + '|';
        });
        header += '|';
    }
    else {
        header += '||';
    }
    return header + output;
}
exports.compress = compress;
/**
 * Transforma uma string comprimida em um flatten object
 *
 * STRINGS||KEYS_VALUES||INDEXES
 * @param {*} strings
 */
function decompress(compressed) {
    var step = 1;
    var char;
    var value;
    var indexStart = 0;
    var indexReal;
    var isKey = false;
    var json = '';
    var markedAsString = [];
    var keysValues = DEFAULT_KEYS.slice(0);
    var string = '';
    for (var index = 0, l = compressed.length; index < l; index++) {
        char = compressed.charAt(index);
        if (step === 1) {
            markedAsString.push(toIndex(char));
        }
        else if (char === '|') {
            keysValues.push(string);
            string = '';
        }
        else {
            string += char;
        }
        // Fechamento de parte (||)
        if (char === '|' && compressed.charAt(index + 1) === '|') {
            step++;
            index++;
            // INDEXES
            if (step === 3) {
                index++;
                indexStart = index;
                // Loop interno para o passo 3, evita os IF's acima 
                for (; index < l; index++) {
                    char = compressed.charAt(index);
                    if (char === OBJECT_OPEN) {
                        json += '{';
                        isKey = true;
                        continue;
                    }
                    if (char === OBJECT_CLOSE) {
                        // js.substring , 
                        json = json.slice(0, -1) + '},';
                        isKey = compressed.charAt(index + 1) !== OBJECT_CLOSE;
                        continue;
                    }
                    indexReal = index - indexStart;
                    value = keysValues[toIndex(char)];
                    // Se está determinado que o valor é string, ignora formatação de primitivos
                    var parsePrimitive = markedAsString.indexOf(indexReal) < 0;
                    if (typeof value === 'string') {
                        if (!isKey && parsePrimitive) {
                            if (value.match(/^[-+]?\d+$/)) {
                                value = Number.parseInt(value);
                            }
                            else if (value.match(/^[-+]?[\d.,]+$/)) {
                                value = Number.parseFloat(value);
                            }
                            else if (['true', 'false', 'null'].indexOf(value) >= 0) {
                                value = PRIMITIVES[value];
                            }
                            else {
                                value = '"' + value + '"';
                            }
                        }
                        else {
                            value = '"' + value + '"';
                        }
                    }
                    json += value;
                    if (isKey) {
                        json += ':';
                        isKey = false;
                    }
                    else {
                        isKey = true;
                        json += ',';
                    }
                }
            }
        }
    }
    if (json.charAt(json.length - 1) === ',') {
        json = json.slice(0, -1);
    }
    return JSON.parse(json);
}
exports.decompress = decompress;
//# sourceMappingURL=Compress.js.map