import { AnyObject } from '../Constants';
export declare const ErrorIdRequired = "\u00C9 necess\u00E1rio que todos os itens de um Array de Objetos tenham identificador (id, _id, key ou _key)";
export declare const ErrorArrayMixedNotAccepted = "N\u00E3o \u00E9 permitido o uso de Arrays mistos (Objetos e Primitivos)";
export declare const ErrorDeltaCompressHasNoDiff = "N\u00E3o \u00E9 poss\u00EDvel comprimir o objeto. N\u00E3o existe diferen\u00E7as. Consultar Delta.hasDiff() antes da invoca\u00E7\u00E3o deste m\u00E9todo";
export declare const ErrorDeltaBadInitialization = "Objeto de delta inicializado de forma errada. Consulte Delta::diff e Delta::decompress";
export declare const ErrorOnlyObjectOrArrayAccepted = " DTO aceita apenas Objetos e Arrays como entrada.";
export declare const ErrorDecompressAcceptOnlyStrings = " DTO::decompress() aceita apenas String como entrada.";
export declare const ErrorArrayMultidimensionalNotAccepted = "N\u00E3o \u00E9 permitido o uso de array multidimensional. Ex. key=[[],[]]";
/**
 * Data Transfer Object
 *
 * Utilitário para padronizar o transporte de dados pela rede.
 *
 * O DTO pode fazer diff e patch do objeto além de permitir comprimir os dados para transporte
 */
export default class DTO {
    /**
     * Obtém um objeto DTO a partir de um valor comprimido
     *
     * @param compressed
     */
    static decompress(compressed: string): DTO;
    /**
     * Obtém um objeto DTO a partir de um valor modificado
     *
     * @param flattened
     */
    static flattened(flattened: any): DTO;
    private object;
    private flattened;
    private compressed?;
    constructor(object: AnyObject);
    /**
    * Obtém a versão modificada do objeto original
    */
    flatten(): AnyObject;
    /**
     * Obtém a versão do objeto original
     */
    unflatten(): AnyObject;
    /**
     * Gera uma versão comprimida deste objeto
     */
    compress(): string;
    /**
     * Permite obter a diferença entre o objeto atual e outro objeto
     *
     * @param other
     */
    diff(other: DTO): Delta;
    /**
     * Aplica um patch a esse objeto, retornando uma nova instancia do DTO
     *
     * @param delta
     */
    patch(delta: Delta): DTO;
    /**
    * Aplica um patch (compactado) a esse objeto, retornando uma nova instancia do DTO
    *
    * @param delta
    */
    patchCompressed(compressedDelta: string): DTO;
    /**
     * Remover todos os arrays de um objeto, mapeando-os por [id|_id|key|_key] em caso de objetos e INDEX para dados simples
     *
     * Valores UNDEFINED são descartados
     *
     * @param {*} obj
     */
    private flattenObject;
    /**
     * Faz a decodificação do objeto anterior
     *
     * @param {*} obj
     */
    private unflattenObject;
}
/**
 * Definição do objeto de diff
 */
export interface DeltaObject {
    /**
     * Diferenças entre os objetos
     */
    d?: AnyObject;
    /**
     * Atributos adicionados
     */
    a?: AnyObject;
    /**
     * Atributos modificados
     */
    m?: AnyObject;
    /**
     * Alterações recursivas
     */
    r?: AnyObject;
}
/**
 * Representação da diferença entre dois objetos DTO
 */
export declare class Delta {
    private deltaObj?;
    private compressed?;
    private areEquals?;
    /**
     * Obtém um objeto Delta a partir da comparação de dois objetos DTO
     *
     * @param compressed
     */
    static diff(dtoA: DTO, dtoB: DTO): Delta;
    /**
     * Obtém um objeto Delta a partir de um valor comprimido
     *
     * @param compressedDelta
     */
    static decompress(compressedDelta: string): Delta;
    /**
     * Informa que não existe diferença entre os dois objetos testados
     */
    hasDiff(): boolean;
    /**
     * Obtém a versão comprimida do DIFF, usada para tranporte
     */
    compress(): string;
    /**
     * Obtém o objeto delta desta instancia
     */
    delta(): DeltaObject;
    /**
     * Aplica esse Delta no outro objeto
     *
     * @param old
     */
    patch(old: DTO): DTO;
    /**
     * Obtém a diferença entre dois objetos (Baseado no dffptch com modificações).
     *
     * @link https://github.com/paldepind/dffptch
     *
     * @param a
     * @param b
     */
    private diffObject;
    /**
     * Aplica o delta em um objeto (Baseado no dffptch com modificações)
     *
     * @link https://github.com/paldepind/dffptch
     * @param flattened
     * @param delta
     */
    private patchObject;
}
