"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var LEVELS = {
    'TRACE': 0,
    'DEBUG': 1,
    'INFO': 2,
    'WARN': 3,
    'ERROR': 4,
    'OFF': 5
};
var NAMED = {
    'TRACE': 'TRACE',
    'DEBUG': 'DEBUG',
    'INFO': 'INFO ',
    'WARN': 'WARN ',
    'ERROR': 'ERROR',
    'OFF': ''
};
var METHODS = {
    'TRACE': console.log,
    'DEBUG': console.debug,
    'INFO': console.info,
    'WARN': console.warn,
    'ERROR': console.error,
    'OFF': function () { }
};
/**
 * Biblioteca simples de Log
 */
var Logger = /** @class */ (function () {
    function Logger(name) {
        this.level = 'INFO';
        this.levelValue = LEVELS['INFO'];
        this.levelName = NAMED['INFO'];
        this.name = name;
    }
    Logger.get = function (name) {
        if (!Logger.LOGS[name]) {
            Logger.LOGS[name] = new Logger(name);
        }
        return Logger.LOGS[name];
    };
    Logger.prototype.trace = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        this.log('TRACE', args);
    };
    Logger.prototype.debug = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        this.log('DEBUG', args);
    };
    Logger.prototype.info = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        this.log('INFO', args);
    };
    Logger.prototype.warn = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        this.log('WARN', args);
    };
    Logger.prototype.error = function () {
        var args = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            args[_i] = arguments[_i];
        }
        this.log('ERROR', args);
    };
    Logger.prototype.setLevel = function (level) {
        this.level = level;
        this.levelValue = LEVELS[level];
        this.levelName = NAMED[level];
    };
    Logger.prototype.getLevel = function () {
        return this.level;
    };
    Logger.prototype.isEnabled = function (level) {
        return LEVELS[level] >= this.levelValue;
    };
    Logger.prototype.log = function (level, args) {
        var _this = this;
        if (!this.isEnabled(level)) {
            return;
        }
        var date = new Date();
        var out = [date.toISOString() + ' ' + this.levelName + ' ' + this.name + ' - '];
        args.forEach(function (arg) {
            if (arg instanceof Error) {
                out.push(arg.message);
                if (arg.stack) {
                    out.push('\n' + arg.stack);
                }
            }
            else if (arg instanceof Date) {
                out.push(arg.toISOString());
            }
            else if (arg === undefined) {
                out.push("undefined");
            }
            else if (typeof arg === 'object') {
                out.push(JSON.stringify(arg, _this.getCircularReplacer(), 2));
            }
            else {
                out.push(arg + '');
            }
        });
        METHODS[level](out.join(''));
    };
    /**
     * https://developer.mozilla.org/en-US/docs/Web/JavaScript/Reference/Errors/Cyclic_object_value
     */
    Logger.prototype.getCircularReplacer = function () {
        var seen = new WeakSet();
        return function (key, value) {
            if (typeof value === "object"
                && value !== null
                && !(value instanceof Boolean)
                && !(value instanceof Date)
                && !(value instanceof Number)
                && !(value instanceof RegExp)
                && !(value instanceof String)) {
                if (seen.has(value)) {
                    return '';
                }
                seen.add(value);
            }
            return value;
        };
    };
    ;
    Logger.LOGS = {};
    Logger.mainLogger = Logger.get('main');
    return Logger;
}());
exports.default = Logger;
//# sourceMappingURL=Logger.js.map