import React from "react";
/**
 * Classe utilitária para facilitar a observação de resolução de promises dentro de um componente
 *
 * Ex.:
 * observerLogin: PromiseObserver = new PromiseObserver(this);
 * ...
 * this.observerLogin.observe<Usuario>(LocalStorage.get('usuario')) // O estado foi atualizado
 *     .then((usuario) => {
 *        if (usuario && usuario._id) {
 *           SessaoUsuario.start(usuario as Usuario);
 *           // Está logado
 *           this.props.navigation.replace('Inicio');
 *        }
 *     });
 * ...
 * <Button
 *      isLoading={this.observerLogin.isLoding} // Bloqueio de campo até que o promise seja resolvido
 *      reverse={true}
 *      title="Entrar"
 *      onPress={this.efetuarLogin}
 *      disabled={this.state.usuario === ''}
 *      style={{ marginTop: 10 }}
 *   />
 */
export default class ReactIsLoading {
    /**
     * Informa que é o primeiro carregamento observado por essa instancia
     */
    isFirstLoding: boolean;
    /**
     * Informa se existe promise sendo processado agora
     */
    isLoding: boolean;
    /**
     * Informa se possui algum erro
     */
    hasError: boolean;
    /**
     * Hash usado para permitir usar como ExtraData em listas, afim de forçar renderização de PureComponents
     */
    hash: number;
    /**
     * O último erro capturado
     */
    error: any;
    private componentUnmounted;
    private component;
    private promises;
    private hashName;
    constructor(component: React.Component);
    /**
     * Quando o promise for resolvido, altera o estado do componente, permitindo uma nova renderização do mesmo
     *
     * O promise será finalizado antes de o componente informar o status (isLoading = true)
     *
     * @param promise
     */
    observe<T>(promise: Promise<T>): Promise<T>;
    private newState;
}
